@@ -676,6 +676,44 @@ inline ZZ& operator%=(ZZ& x, const ZZ& b)
    { rem(x, x, b); return x; } 
 
 
+// preconditioned single-precision variant
+// not documented for now...
+
+
+#define NTL_ZZ_reduce_struct_DefaultSize (64)
+// number of zzigits stored in table
+
+
+struct sp_ZZ_reduce_struct_policy {
+
+   static
+   void deleter(_ntl_general_rem_one_struct *pinfo)
+   {
+      _ntl_general_rem_one_struct_delete(pinfo); 
+   }
+
+};
+
+struct sp_ZZ_reduce_struct {
+   long p;
+   UniquePtr<_ntl_general_rem_one_struct,sp_ZZ_reduce_struct_policy> pinfo;
+
+   sp_ZZ_reduce_struct() : p(0) { }
+
+   void build(long _p, long sz = NTL_ZZ_reduce_struct_DefaultSize) 
+   {
+      pinfo.reset(_ntl_general_rem_one_struct_build(_p, sz));
+      p = _p;
+   }
+
+   long rem(const ZZ& a) const
+   {
+      return _ntl_general_rem_one_struct_apply(a.rep, p, pinfo.get());
+   }
+};
+
+
+
 /**********************************************************
 
                         GCD's
